/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-settings.h"

#define PATH "/apps/openldev"

// List of keys that should be loaded
gchar *keys [] = { "/auto_save", "/browser_width", "/output_height", "/width", "/height",
                   "/undo_actions", "/tab_size", "/auto_indent", "/auto_load", "/brackets",
                   "/line_numbers", "/replace_tabs", "/right_margin", "/wrap_enable",
                   "/wrap_split", "/error_color", "/warning_color", "/font_face",
                   "/font_size", "/last_project", "/recent_files", "/plugins" };

// The corresponding numbers to the keys
enum 
{ 
  AUTO_SAVE=0, BROWSER_WIDTH, OUTPUT_HEIGHT, WINDOW_WIDTH, WINDOW_HEIGHT,
  UNDO_ACTIONS, TAB_SIZE, AUTO_INDENT, AUTO_LOAD, BRACKETS, LINE_NUMBERS,
  REPLACE_TABS, RIGHT_MARGIN, WRAP_ENABLE, WRAP_SPLIT, ERROR_COLOR,
  WARNING_COLOR, FONT_FACE, FONT_SIZE, LAST_PROJECT, RECENT, PLUGINS 
};

static void openldev_environment_load_settings_file (EnvironmentSettings*);
static void openldev_environment_create_all (EnvironmentSettings*);
static void openldev_environment_set_gboolean (EnvironmentSettings*, gint, gboolean);
static void openldev_environment_set_string (EnvironmentSettings*, gint, gchar*);
static void openldev_environment_set_int (EnvironmentSettings*, gint, gint);

static gint openldev_environment_get_int (EnvironmentSettings*, gint);
static gboolean openldev_environment_get_bool (EnvironmentSettings*, gint);
static gchar* openldev_environment_get_string (EnvironmentSettings*, gint);
static GSList* openldev_environment_get_list (EnvironmentSettings*, gint);

// Create a new environment settings structure
EnvironmentSettings* 
openldev_environment_settings_new ()
{
  EnvironmentSettings* settings = new EnvironmentSettings();
  
  settings->client = gconf_client_get_default ();
  openldev_environment_load_settings_file (settings);
  
  return settings;
}

// Load all of the environment settings from GConf
static void 
openldev_environment_load_settings_file (EnvironmentSettings *settings)
{
  if (!gconf_client_dir_exists (settings->client, PATH, NULL))
    gconf_client_add_dir (settings->client, PATH, GCONF_CLIENT_PRELOAD_NONE, NULL);
  
  openldev_environment_create_all (settings);
  
  settings->auto_save = openldev_environment_get_int (settings, 0);
  settings->sidebar_width = openldev_environment_get_int (settings, 1);
  settings->outputmanager_height = openldev_environment_get_int (settings, 2);
  settings->width = openldev_environment_get_int (settings, 3);
  settings->height = openldev_environment_get_int (settings, 4);
  settings->undo = openldev_environment_get_int (settings, 5);
  settings->tab_size = openldev_environment_get_int (settings, 6);
   
  settings->auto_indent = openldev_environment_get_bool (settings, 7);
  settings->auto_load = openldev_environment_get_bool (settings, 8);
  settings->brackets = openldev_environment_get_bool (settings, 9);
  settings->line_numbers = openldev_environment_get_bool (settings, 10);
  settings->replace_tabs = openldev_environment_get_bool (settings, 11);
  settings->right_margin = openldev_environment_get_bool (settings, 12);
  settings->wrap = openldev_environment_get_bool (settings, 13);
  settings->wrap_split = openldev_environment_get_bool (settings, 14);
   
  settings->error = openldev_environment_get_string (settings, 15);
  settings->warning = openldev_environment_get_string (settings, 16);
  settings->font_face = openldev_environment_get_string (settings, 17);
  settings->font_size = openldev_environment_get_string (settings, 18);
  settings->last_project = openldev_environment_get_string (settings, 19);
  
  settings->recent_files = openldev_environment_get_list (settings, 20);
  settings->plugins = openldev_environment_get_list (settings, 21);
}

static int 
openldev_environment_get_int (EnvironmentSettings *settings,
                              gint i)
{ 
  return gconf_client_get_int (settings->client, g_strconcat (PATH, keys[i], NULL), NULL);
}

static gboolean 
openldev_environment_get_bool (EnvironmentSettings *settings, 
                               gint i)
{ 
  return gconf_client_get_bool (settings->client, g_strconcat (PATH, keys[i], NULL), NULL);
}

static gchar* 
openldev_environment_get_string (EnvironmentSettings *settings, 
                                 gint i)
{
  return gconf_client_get_string (settings->client, g_strconcat (PATH, keys[i], NULL), NULL);
}

static GSList* 
openldev_environment_get_list (EnvironmentSettings *settings, 
                               gint i)
{
  return gconf_client_get_list (settings->client, g_strconcat (PATH, keys[i], NULL), 
                                GCONF_VALUE_STRING, NULL);
}

// Make sure that all of the settings already exist in GConf. If they don't, the
// default value is used.
static void 
openldev_environment_create_all (EnvironmentSettings *settings)
{
  gint i;
  GConfValue *value;
  
  GSList *list = NULL;
  gint default_int[] = { 10, 175, 175, 800, 600, 25, 2 };
  gboolean default_gboolean[] = { TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, TRUE, FALSE };
  gchar *default_str[] = { "#FF0000", "#0000CC", "Monospace", "10", "EMPTY" };
  
  for (i = 0; i < 22; i++)
  {
    gchar *key = g_strconcat (PATH, keys[i], NULL);
    value = gconf_client_get (settings->client, key, NULL);
    
    if (value == NULL)
    {
      gconf_client_add_dir (settings->client, key, GCONF_CLIENT_PRELOAD_NONE, NULL);
      if (i < 7)
      	gconf_client_set_int (settings->client, key, default_int[i], NULL);
      else if (i < 15)
        gconf_client_set_bool (settings->client, key, default_gboolean[i-7], NULL);
      else if (i < 20)
        gconf_client_set_string (settings->client, key, default_str[i-15], NULL);
      else
      {
        if (i == 21)
        {
          list = g_slist_append (list, (gpointer) "makefileeditor.plugin");
          list = g_slist_append (list, (gpointer) "insertinfo.plugin");
        }
        
        gconf_client_set_list (settings->client, key, GCONF_VALUE_STRING, list, NULL);
      }
    }
    
    g_free (key);
  }
}

// These three functions set values of their type and then suggest
// as sync to GConf to make sure the changes are written.
static void 
openldev_environment_set_gboolean (EnvironmentSettings *settings,
                                   gint id, 
                                   gboolean value)
{
  gconf_client_set_bool (settings->client, g_strconcat (PATH, keys[id], NULL), 
                         value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

static void 
openldev_environment_set_string (EnvironmentSettings *settings, 
                                 gint id, 
                                 gchar *value)
{
  gconf_client_set_string (settings->client, g_strconcat (PATH, keys[id], NULL), 
                           value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

static void 
openldev_environment_set_int (EnvironmentSettings *settings, 
                              gint id, 
                              gint value)
{
  gconf_client_set_int (settings->client, g_strconcat (PATH, keys[id], NULL), 
                        value,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

static void 
openldev_environment_set_list (EnvironmentSettings *settings, 
                               gint id, 
                               GSList *list)
{
  gconf_client_set_list (settings->client, g_strconcat (PATH, keys[id], NULL), 
                         GCONF_VALUE_STRING, list,  NULL);
  gconf_client_suggest_sync (settings->client, NULL);
}

// Functions that allow the user to change environment settings
void 
openldev_environment_set_auto_indent (EnvironmentSettings* settings, 
                                      gboolean value)
{
  settings->auto_indent = value;
  openldev_environment_set_gboolean (settings, AUTO_INDENT, value);
}

void 
openldev_environment_set_auto_load (EnvironmentSettings* settings, 
                                    gboolean value)
{
  settings->auto_load = value;
  openldev_environment_set_gboolean (settings, AUTO_LOAD, value);
}

void 
openldev_environment_set_brackets (EnvironmentSettings* settings, 
                                   gboolean value)
{
  settings->brackets = value;
  openldev_environment_set_gboolean (settings, BRACKETS, value);
}

void 
openldev_environment_set_line_numbers (EnvironmentSettings* settings, 
                                       gboolean value)
{
  settings->line_numbers = value;
  openldev_environment_set_gboolean (settings, LINE_NUMBERS, value);
}

void 
openldev_environment_set_replace_tabs (EnvironmentSettings* settings, 
                                       gboolean value)
{
  settings->replace_tabs = value;
  openldev_environment_set_gboolean (settings, REPLACE_TABS, value);
}

void 
openldev_environment_set_right_margin (EnvironmentSettings* settings, 
                                       gboolean value)
{
  settings->right_margin = value;
  openldev_environment_set_gboolean (settings, RIGHT_MARGIN, value);
}

void 
openldev_environment_set_wrap (EnvironmentSettings* settings, 
                               gboolean value)
{
  settings->wrap = value;
  openldev_environment_set_gboolean (settings, WRAP_ENABLE, value);
}

void 
openldev_environment_set_wrap_split (EnvironmentSettings* settings, 
                                     gboolean value)
{
  settings->wrap_split = value;
  openldev_environment_set_gboolean (settings, WRAP_SPLIT, value);
}

void 
openldev_environment_set_auto_save (EnvironmentSettings* settings, 
                                    gint value)
{
  settings->auto_save = value;
  openldev_environment_set_int (settings, AUTO_SAVE, value);
}

void 
openldev_environment_set_sidebar_width (EnvironmentSettings* settings, 
                                        gint value)
{
  settings->sidebar_width = value;
  openldev_environment_set_int (settings, BROWSER_WIDTH, value);
}

void 
openldev_environment_set_height (EnvironmentSettings* settings, 
                                 gint value)
{
  settings->height = value;
  openldev_environment_set_int (settings, WINDOW_HEIGHT, value);
}

void 
openldev_environment_set_outputmanager_height (EnvironmentSettings* settings, 
                                               gint value)
{
  settings->outputmanager_height = value;
  openldev_environment_set_int (settings, OUTPUT_HEIGHT, value);
}

void 
openldev_environment_set_tab_size (EnvironmentSettings* settings, 
                                   gint value)
{
  settings->tab_size = value;
  openldev_environment_set_int (settings, TAB_SIZE, value);
}

void 
openldev_environment_set_undo (EnvironmentSettings* settings, 
                               gint value)
{
  settings->undo = value;
  openldev_environment_set_int (settings, UNDO_ACTIONS, value);
}

void 
openldev_environment_set_width (EnvironmentSettings* settings, 
                                gint value)
{
  settings->width = value;
  openldev_environment_set_int (settings, WINDOW_WIDTH, value);
}

void 
openldev_environment_set_error (EnvironmentSettings* settings, 
                                gchar* value)
{
  g_free (settings->error);
  settings->error = NULL;
  settings->error = g_strdup (value);;
  openldev_environment_set_string (settings, ERROR_COLOR, value);
}

void 
openldev_environment_set_font_face (EnvironmentSettings* settings, 
                                    gchar* value)
{
  g_free (settings->font_face);
  settings->font_face = NULL;
  settings->font_face = g_strdup (value);
  openldev_environment_set_string (settings, FONT_FACE, value);
}

void 
openldev_environment_set_font_size (EnvironmentSettings* settings, 
                                    gchar* value)
{
  g_free (settings->font_size);
  settings->font_size = NULL;
  settings->font_size = g_strdup (value);
  openldev_environment_set_string (settings, FONT_SIZE, value);
}

void 
openldev_environment_set_last_project (EnvironmentSettings* settings, 
                                       gchar* value)
{
  g_free (settings->last_project);
  settings->last_project = NULL;
  settings->last_project = g_strdup (value);
  openldev_environment_set_string (settings, LAST_PROJECT, value);
}

void 
openldev_environment_set_warning (EnvironmentSettings* settings, 
                                  gchar* value)
{
  g_free (settings->warning);
  settings->warning = NULL;
  settings->warning = g_strdup (value);
  openldev_environment_set_string (settings, WARNING_COLOR, value);
}

void 
openldev_environment_add_recent_file (EnvironmentSettings *settings, 
                                      gchar *value)
{
  GSList *temp;
  gchar *new_value;
  gpointer data;

  temp = settings->recent_files;
  while (temp != NULL)
  {
    if (g_ascii_strcasecmp (value, (gchar*) temp->data) == 0)
      return;
    temp = temp->next;
  }
  
  new_value = g_strdup (value);
  settings->recent_files = g_slist_prepend (settings->recent_files, (gpointer) new_value);
  
  if (g_slist_length (settings->recent_files) > 5)
  {
    data = g_slist_nth_data (settings->recent_files, 5);
    settings->recent_files = g_slist_remove (settings->recent_files, data);
  }
  
  openldev_environment_set_list (settings, RECENT, settings->recent_files);
}

gchar* 
openldev_environment_get_recent_short (EnvironmentSettings *settings,
                                       gint item)
{
  GString *temp = g_string_new ((gchar*) g_slist_nth_data (settings->recent_files, item));
  gchar *find;
  gsize len;
  
  /* Parse through the string, splitting it into a GList at the '/' character. */
  while (temp->str[0] == '/')
  {
    while (temp->str[0] == '/')
      g_string_erase (temp, 0, 1);
    find = g_strstr_len (temp->str, temp->len, "/");
    
    if (find != NULL)
    {
      len = (gsize) (strlen (temp->str) - strlen (find));
      g_string_erase (temp, 0, len);
    }
  }

  return temp->str;
}

void 
openldev_environment_remove_plugin (EnvironmentSettings *settings, 
                                    gchar *plugin)
{
  GSList *temp;

  temp = settings->plugins;
  while (temp != NULL)
  {
    if (g_ascii_strcasecmp (plugin, (gchar*) temp->data) == 0)
      settings->plugins = g_slist_remove (settings->plugins, temp->data);
    temp = temp->next;
  }

  openldev_environment_set_list (settings, PLUGINS, settings->plugins);
}

void 
openldev_environment_add_plugin (EnvironmentSettings *settings, 
                                 gchar *plugin)
{
  GSList *temp;
  gchar *new_value;

  temp = settings->plugins;
  while (temp != NULL)
  {
    if (g_ascii_strcasecmp (plugin, (gchar*) temp->data) == 0)
      return;
    temp = temp->next;
  }

  new_value = g_strdup (plugin);
  settings->plugins = g_slist_prepend (settings->plugins, (gpointer) new_value);
  openldev_environment_set_list (settings, PLUGINS, settings->plugins);
}

gboolean 
openldev_environment_get_plugin (EnvironmentSettings *settings, 
                                 gchar *name)
{
  GSList *temp;
  gchar *new_value;

  temp = settings->plugins;
  while (temp != NULL)
  {
    if (g_ascii_strcasecmp (name, (gchar*) temp->data) == 0)
      return TRUE;
    temp = temp->next;
  }

  return FALSE;
}
