/***************************************************************************
 *   Copyright (C) 2006 by Andrew Krause                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-utils.h"
#include "openldev-gettext.h"

#include <glade/glade.h>

gchar* 
openldev_add_prefix (gchar *text)
{ 
  return g_strdup_printf ("%s%s", INSTALL_PREFIX, text);
}

// Throw an error dialog with the specified message
void 
openldev_error (string message)
{
  string gladefile = openldev_add_prefix ("/share/openldev/glade/message.glade");
  GladeXML *xml = glade_xml_new (gladefile.c_str(), NULL, NULL);
  GtkWidget *dialog = glade_xml_get_widget (xml, "dialog");
  GtkWidget *label = glade_xml_get_widget (xml, "label");
  
  gtk_label_set_markup (GTK_LABEL (label), message.c_str());
  int result = gtk_dialog_run (GTK_DIALOG (dialog));
  
  switch (result)
  {
    default:
      gtk_widget_destroy (dialog);
  }
}

// Show a question dialog with the specified message. Return the answer the
// user choses. This function handles creating the dialog, setting the properties
// and then deleting it and returning the answer.
gboolean 
openldev_question (string question)
{
  GtkWidget *dialog = gtk_dialog_new (),
            *yes = gtk_button_new_from_stock ("gtk-yes"),
            *no = gtk_button_new_from_stock ("gtk-no"),
            *image = gtk_image_new_from_stock ("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG),
            *label = gtk_label_new (question.c_str()),
            *hbox = gtk_hbox_new (FALSE, 10);
  
  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
  gtk_widget_set_size_request (label, 300, -1);
  gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
  gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  
  gtk_box_pack_start (GTK_BOX(GTK_DIALOG (dialog)->vbox),
                      hbox, TRUE, TRUE, 0);
  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 7);
  gtk_window_set_title (GTK_WINDOW (dialog), _("Question"));  
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), TRUE);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), no, -9);
  gtk_dialog_add_action_widget (GTK_DIALOG (dialog), yes, -8);
  
  gtk_widget_show_all (dialog);
  switch (gtk_dialog_run (GTK_DIALOG (dialog)))
  {
    case -8:
    {
      gtk_widget_destroy (dialog);
      return TRUE;
      break;
    }
    default:
    {
      gtk_widget_destroy (dialog);
      return FALSE;
      break;
    }
  }
}

// Write the text to the file. This function receives the filename and the file
// text. The text is then written to the file located at the filename.
void 
openldev_write_file (string filename, 
                     string text)
{
  ofstream fout (filename.c_str());
  fout << text;
  fout.close ();
}

// Read in the text from a file. This function receives a filename and returns
// the text string of the file.
gchar* 
openldev_read_file (gchar *filename)
{
  gchar *contents;
  gsize length;
  GError *error = NULL;
  
  if (!g_file_test (filename, G_FILE_TEST_EXISTS))
  {
    openldev_error (_("Error: File does not exist!"));
    return NULL;
  }
  
  if (g_file_get_contents (filename, &contents, &length, &error));
    return contents;

  openldev_error (_("Error: There was an error opening this file!"));
  return "";

}

// Check if the output is an error or warning. If it only has two colons in
// close proximity of eachother in the beginning, then it is an error
// or a warning.
gboolean 
openldev_is_error_or_warning (string output)
{
  // Find :'s (It is an error or warning if 2 :'s are found close together
  int location  = output.find (":", 0);
  int location2 = output.find (":", location+1);
  
  // Check if :'s were found & return if they were in proper format
  if (location != (signed int)string::npos)
    if (location2 != (signed int)string::npos && (location2-location < 5))
      return TRUE;
  
  return FALSE;
}

// Get file name and line of the error or warning. This function parses the
// error or warning sent by the output string. Output is then converted into
// only the short filename containing the error & the line number of the
// error or warning is returned.
int 
openldev_get_file_name_and_line (string &output)
{
  string line;
  int location = output.find(":", 0);
  
  // Get the line number & delete it out so only filename is left
  line = output.substr (location+1, output.length()-location-1);
  output.erase (location, output.length()-location);
  
  // Erase all except for the line number
  location = line.find (":", 0);
  line.erase (location, line.length()-location);
  
  // Return the line number
  return atol (line.c_str());
}

// Create new stock menu item. This function receives the icon factory, the
// name of the item, the id of the item, and the filename of the icon.
void 
openldev_add_new_stock_item (GtkIconFactory *icon_factory, 
                             gchar *name,
                             gchar *id, 
                             gchar *filename)
{
  GtkIconSource *icon_source = gtk_icon_source_new ();
  GtkIconSet *icon = gtk_icon_set_new ();
  
  // Set the properties of the stock menu/toolbar item
  gtk_icon_source_set_icon_name (icon_source, name);
  gtk_icon_source_set_filename (icon_source, filename);
  gtk_icon_set_add_source (icon, icon_source);
  gtk_icon_factory_add (icon_factory, id, icon);
}

// Get all of the text in the GtkSourceBuffer and return it.
string 
openldev_file_get_text_in_buffer (GtkSourceBuffer *buffer)
{
  GtkTextIter start, end;
  gtk_text_buffer_get_start_iter (GTK_TEXT_BUFFER (buffer), &start);
  gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (buffer), &end);
  return gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buffer), &start, &end, TRUE);
}

// Create a new process
gint 
openldev_fork_new_process (string command)
{
  gint pid;

  if (pid = fork())
    return -1;
  else
    execl ("/bin/sh", "sh", "-c", command.c_str(), 0);
  
  return pid;
}
