/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-output-build.h"
#include "../openldev-gettext.h"

void erase (OutputBuild*);
void add_row (OutputBuild*, string, string);
void update_output (OutputBuild*);

static void output_build_class_init (OutputBuildClass *klass);
static void output_build_init (OutputBuild *build);

GType
output_build_get_type ()
{
  static GType build_type = 0;

  if (!build_type)
  {
    static const GTypeInfo build_info =
    {
	    sizeof (OutputBuildClass),
	    NULL, NULL,
	    (GClassInitFunc) output_build_class_init,
      NULL, NULL,
      sizeof (OutputBuild), 0,
	    (GInstanceInitFunc) output_build_init,
    };

    build_type = g_type_register_static (GTK_TYPE_TREE_VIEW, "OutputBuild", 
                                         &build_info, (GTypeFlags) 0);
  }

  return build_type;
}

static void
output_build_class_init (OutputBuildClass *klass)
{
}

static void
output_build_init (OutputBuild *build)
{
  build->swin = gtk_scrolled_window_new (NULL, NULL);
  
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (build), FALSE);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (build->swin),
                                         GTK_WIDGET (build));
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (build->swin),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
                                  
  PangoFontDescription *fd = pango_font_description_from_string ("Monospace 9");
  gtk_widget_modify_font (GTK_WIDGET (build), fd);
}

GtkWidget*
output_build_new ()
{
  return GTK_WIDGET (g_object_new (output_build_get_type (), NULL));
}

GtkWidget* 
output_build_get_scrolled_window (OutputBuild *build)
{ return build->swin;
}

// Run the specified command and print the output to the build output treeview
gboolean 
output_build_run_command (OutputBuild *build,
                          const char* command,
                          EnvironmentSettings *env)
{
  gchar buff[512];
  FILE *in;
  gboolean successful = TRUE, autogenerate = FALSE;
  int location;
  string text, line, comm = command;
  GtkTreeIter iter;
  GTimer *timer = g_timer_new ();
  
  g_timer_start (timer);
  erase (build);
  add_row (build, comm.substr (0, comm.find ("2>&1", 0)), "#000000");
  
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (build));
  gtk_tree_model_get_iter (build->model, &iter, gtk_tree_path_new_from_string ("0"));
  gtk_tree_selection_select_iter (selection, &iter);
    
  // Open pipe & loop until the program is complete
  in = popen (comm.c_str(), "r");
  
  while (fgets (buff, sizeof (buff), in) && build->continue_build)
  {
    text += buff;
    // Find the endl's & split them to be added to build_box
    location = text.find ("\n", 0);
    while (location != (signed int) string::npos) 
    {     
      // Check to see if autogen.sh must be run before build
      if (text.find ("No targets specified", 0) != string::npos)
        autogenerate = TRUE;
        
      // Append line & erase it from the buffer
      line = text.substr (0, location);
      text.erase (0, location + 1);
      location = text.find ("\n", 0);

      // Apply Error or Warning colors
      if (line.find ("error: ", 0) != string::npos ||
          (line.find ("***", 0) != string::npos && 
           line.find ("Error ", 0) != string::npos) ||
          line.find ("undefined reference", 0) != string::npos) 
      {
        add_row (build, line, env->error);
        successful = FALSE;
      }
      else if (line.find ("***", 0) != string::npos && 
               line.find ("Error ", 0) != string::npos)
      {
        add_row (build, line, env->error);
        successful = FALSE;
      }
      else if (line.find (": ", 0) != string::npos)
        add_row (build, line, env->warning);
      else
        add_row (build, line, "#000000");
      
      // Update the build_box when there is output
      update_output (build);
    }
  }
    
  // Close the stream, & print whether command was successful or not
  in = NULL;
  g_timer_stop (timer);
  if (successful && build->continue_build)
    add_row (build, _("Completed ... Successful"), "#000000");
  else if (!build->continue_build)
    add_row (build, _("Incomplete ... Terminated"), env->error);
  else
    add_row (build, _("Completed ... Failed"), env->error);
  
  char time[10];
  snprintf (time, sizeof (time), "%f", g_timer_elapsed (timer, NULL));
  string ttime = time;
  if (ttime.find (".", 0) != string::npos)
    ttime= ttime.substr (0, ttime.find (".", 0));
  if (ttime == "0")
    ttime = "1";
  
  add_row (build, string (_("Total time elapsed: ") + ttime + _(" second(s)")), env->warning);
  
  GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (build->swin));
  gtk_adjustment_set_value (vadj, vadj->upper);
  gtk_widget_set_scroll_adjustments (build->swin, NULL, vadj);
  
  // Return FALSE if the user must run autogen.sh before building & then do it
  if (autogenerate)
    return FALSE;
  
  return TRUE;
}

// Stop any processes currently occuring
void 
output_build_stop_build (OutputBuild *build)
{ 
  build->continue_build = FALSE;
}

void 
erase (OutputBuild *build)
{
  // Clean up build_box & get it ready to accept output
  output_build_clear (build);
  build->model = GTK_TREE_MODEL (gtk_list_store_new (4, G_TYPE_STRING, GDK_TYPE_COLOR, 
                                 G_TYPE_INT, G_TYPE_INT));
  gtk_tree_view_set_model (GTK_TREE_VIEW (build), build->model);
  
  // Append the columns & create the cell renderer
  build->renderer = gtk_cell_renderer_text_new ();
	build->column = gtk_tree_view_column_new_with_attributes (_("Build Results"), 
                  build->renderer, "text", 0, "foreground-gdk", 1, "wrap-mode", 2,
                  "wrap-width", 3, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (build), build->column);

  build->continue_build = TRUE;
}

// Add a row to the build output
void 
add_row (OutputBuild *build,
         string text, 
         string color)
{
  if (text.find ("make  all-recursive", 0) == string::npos)
  {
    GtkTreeIter iter;
    GdkColor tmp = { 0, 0, 0, 0, },
             *gcolor = gdk_color_copy (&tmp);
  
    // Add the command to the box
    gdk_color_parse (color.c_str(), gcolor);
    gtk_list_store_append (GTK_LIST_STORE (build->model), &iter);
	  gtk_list_store_set (GTK_LIST_STORE (build->model), &iter, 
                        0, text.c_str(), 
                        1, gcolor,
                        2, PANGO_WRAP_WORD,
                        3, GTK_WIDGET (build)->allocation.width-10, -1);
  }
}

// Update the build output to enable scrolling to the last row
void 
update_output (OutputBuild *build)
{
  GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (build->swin));
  gtk_adjustment_set_value (vadj, vadj->upper);
  gtk_widget_set_scroll_adjustments (build->swin, NULL, vadj);
  
  while (gtk_events_pending())
    gtk_main_iteration ();
}

void 
output_build_clear (OutputBuild *build)
{
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW (build));
  if (GTK_IS_TREE_MODEL (model))
    if (gtk_tree_model_get_n_columns (model) > 0)
      gtk_tree_view_remove_column (GTK_TREE_VIEW (build),
        gtk_tree_view_get_column (GTK_TREE_VIEW (build), 0));
}
