/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev.h"
#include "openldev-gettext.h"

#define OPENLDEV_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), OPENLDEV_TYPE, OpenLDevPrivate))

typedef struct _OpenLDevPrivate  OpenLDevPrivate;

struct _OpenLDevPrivate 
{
  /*! A GtkNotebook that contains the file browser widget by default. The Makefile Editor plugin also uses this for a display. */
  GtkWidget *sidebar;
  /*! A GtkNotebook that contains all of the open files. This is also capable of handling other types of widgets instead of GtkTextBuffer. */
  GtkWidget *files;
  /*! A GtkNotebook that contains the file build output, search output and terminal widgets by default. It is shown right above the status bar by default. */
  GtkWidget *output;
  /*! The search toolbar shown just above the status bar. It allows the user to search in the current file or search in all source files. */
  FindToolbar *findtoolbar;
  /*! The goto line toolbar shown just above the status bar. It allows the user to jump to a specific line number in the current file. */
  GotoLineToolbar *gototoolbar;
  /*! A structure that contains the menubar, toolbar and popup menus for OpenLDev. */
  MenuHandle *menus;
  /*! A GnomeAppBar widget that displays various messages to the user. */
  StatusBar *statusbar;

  /*! A GtkHPaned widget that contains the Browser and FileManager. */
  GtkWidget *hpaned;
  /*! A GtkVBox widget that contains the menubar, toolbar, status bar and vpaned. */
  GtkWidget *vbox;
  /*! A GtkHPaned widget that contains the hpaned and OutputManager. */
  GtkWidget *vpaned;
  
  /*! General properties of the editor such as what widgets are shown or hidden. */
  EditorProperties    *editor;
  /*! Environment settings specified by the user that control OpenLDev. */
  EnvironmentSettings *env;
  /*! Project settings specified by the user such as build commands. */
  ProjectSettings     *project;
  /*! The list of available and loaded plugins. */
  OpenLDevPluginList  *pluginlist;
};

enum 
{
  FIRST_OPENLDEV_PROPERTY,
  PROP_SIDEBAR,
  PROP_FILES,
  PROP_OUTPUT,
  PROP_HPANED,
  PROP_VBOX,
  PROP_VPANED,
  PROP_FIND_TOOLBAR,
  PROP_GOTO_TOOLBAR,
  PROP_MENU,
  PROP_STATUSBAR,
  PROP_EDITOR,
  PROP_ENV_SETTINGS,
  PROP_PROJECT,
  PROP_PLUGINS
};

enum 
{
  PROJECT_OPEN,
  OPENLDEV_SIGNAL_COUNT
};

// Initialization Functions
void openldev_initialize_objects (OpenLDev*);
void openldev_populate_window (OpenLDev*);
void openldev_create_menus_and_toolbar (OpenLDev*);
void openldev_populate_recent_files (OpenLDev*);

// Signal Handler declarations
void openldev_on_recent_file_activated (GtkMenuItem*, OpenLDev*);

// Class initialization functions
static void openldev_class_init (OpenLDevClass *klass);
static void openldev_init (OpenLDev *openldev);
static void openldev_get_property (GObject*, guint, GValue*, GParamSpec*);
static void openldev_set_property (GObject*, guint, const GValue*, GParamSpec*);

static guint openldev_signals [OPENLDEV_SIGNAL_COUNT] = { 0 };

GType
openldev_get_type (void)
{
  static GType openldev_type = 0;

  if (!openldev_type)
  {
    static const GTypeInfo openldev_info =
    {
	    sizeof (OpenLDevClass),
	    NULL, NULL,
	    (GClassInitFunc) openldev_class_init,
      NULL, NULL,
      sizeof (OpenLDev), 0,
	    (GInstanceInitFunc) openldev_init,
    };

    openldev_type = g_type_register_static (GTK_TYPE_WINDOW, "OpenLDev", 
                                            &openldev_info, (GTypeFlags) 0);
  }

  return openldev_type;
}

static void
openldev_class_init (OpenLDevClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  /* Override the standard functions for setting and retrieving properties. */
  gobject_class->get_property = openldev_get_property;
  gobject_class->set_property = openldev_set_property;

  /* Add GtkIPAddressPrivate as a private data class of GtkIPAddressClass. */
  g_type_class_add_private (klass, sizeof (OpenLDevPrivate));

  /* Register signals. */
  openldev_signals[PROJECT_OPEN] = g_signal_new ("project-open",
					                         G_TYPE_FROM_CLASS (klass),
	                                 G_SIGNAL_RUN_LAST,
	                                 G_STRUCT_OFFSET (OpenLDevClass, openldev),
                                   NULL, NULL,                
					                         g_cclosure_marshal_VOID__VOID,
                                   G_TYPE_NONE, 0);

  /* Add all properties. */
  g_object_class_install_property (gobject_class, PROP_SIDEBAR,
               g_param_spec_object ("sidebar", NULL, NULL, GTK_TYPE_WIDGET,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_FILES,
               g_param_spec_object ("file-browser", NULL, NULL, GTK_TYPE_WIDGET,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_OUTPUT,
               g_param_spec_object ("output-pane", NULL, NULL, GTK_TYPE_WIDGET,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_HPANED,
               g_param_spec_object ("hpaned", NULL, NULL, GTK_TYPE_WIDGET,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_VBOX,
               g_param_spec_object ("vbox", NULL, NULL, GTK_TYPE_WIDGET,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_VPANED,
               g_param_spec_object ("vpaned", NULL, NULL, GTK_TYPE_WIDGET,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_FIND_TOOLBAR,
              g_param_spec_pointer ("find-toolbar", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_GOTO_TOOLBAR,
              g_param_spec_pointer ("goto-toolbar", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_MENU,
              g_param_spec_pointer ("menus", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_STATUSBAR,
              g_param_spec_pointer ("status-bar", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_EDITOR,
              g_param_spec_pointer ("editor-prop", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_ENV_SETTINGS,
              g_param_spec_pointer ("env-settings", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_PROJECT,
              g_param_spec_pointer ("project", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
  
  g_object_class_install_property (gobject_class, PROP_PLUGINS,
              g_param_spec_pointer ("plugins", NULL, NULL,
                                    (GParamFlags) (G_PARAM_READABLE | G_PARAM_WRITABLE)));
}

/* This function is called when the programmer requests the value of a widget
 * property with g_object_get(). */
static void
openldev_get_property (GObject *object,
                       guint prop_id,
                       GValue *value,
                       GParamSpec *pspec)
{
  OpenLDev *openldev = OPENLDEV (object);
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);

  switch (prop_id)
  {
    case PROP_SIDEBAR:      g_value_set_object (value, priv->sidebar); break;
    case PROP_FILES:        g_value_set_object (value, priv->files); break;
    case PROP_OUTPUT:       g_value_set_object (value, priv->output); break;
    case PROP_HPANED:       g_value_set_object (value, priv->hpaned); break;
    case PROP_VBOX:         g_value_set_object (value, priv->vbox); break;
    case PROP_VPANED:       g_value_set_object (value, priv->vpaned); break;
    case PROP_FIND_TOOLBAR: g_value_set_pointer (value, priv->findtoolbar); break;
    case PROP_GOTO_TOOLBAR: g_value_set_pointer (value, priv->gototoolbar); break;
    case PROP_MENU:         g_value_set_pointer (value, priv->menus); break;
    case PROP_STATUSBAR:    g_value_set_pointer (value, priv->statusbar); break;
    case PROP_EDITOR:       g_value_set_pointer (value, priv->editor); break;
    case PROP_ENV_SETTINGS: g_value_set_pointer (value, priv->env); break;
    case PROP_PROJECT:      g_value_set_pointer (value, priv->project); break;
    case PROP_PLUGINS:      g_value_set_pointer (value, priv->pluginlist); break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
    }
}

/* This function is called when the programmer gives a new value for a widget
 * property with g_object_set(). */
static void
openldev_set_property (GObject *object,
                       guint prop_id,
                       const GValue *value,
                       GParamSpec *pspec)
{
  OpenLDev *openldev = OPENLDEV (object);
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  gpointer data;

  switch (prop_id)
  {
    case PROP_SIDEBAR:
      data = g_value_get_object (value); 
      priv->sidebar = GTK_WIDGET (value); 
      break;
    case PROP_FILES:
      data = g_value_get_object (value); 
      priv->files = GTK_WIDGET (value); 
      break;
    case PROP_OUTPUT:
      data = g_value_get_object (value); 
      priv->output = GTK_WIDGET (value); 
      break;
    case PROP_HPANED:
      data = g_value_get_object (value); 
      priv->hpaned = GTK_WIDGET (value); 
      break;
    case PROP_VBOX:
      data = g_value_get_object (value); 
      priv->vbox = GTK_WIDGET (value); 
      break;
    case PROP_VPANED:
      data = g_value_get_object (value); 
      priv->vpaned = GTK_WIDGET (value); 
      break;
    case PROP_FIND_TOOLBAR:
      data = g_value_get_pointer (value); 
      priv->findtoolbar = (FindToolbar*) value; 
      break;
    case PROP_GOTO_TOOLBAR:
      data = g_value_get_pointer (value); 
      priv->gototoolbar = (GotoLineToolbar*) value; 
      break;
    case PROP_MENU:
      data = g_value_get_pointer (value); 
      priv->menus = (MenuHandle*) value; 
      break;
    case PROP_STATUSBAR:
      data = g_value_get_pointer (value); 
      priv->statusbar = (StatusBar*) value; 
      break;
    case PROP_EDITOR:
      data = g_value_get_pointer (value); 
      priv->editor = (EditorProperties*) value;
      break;
    case PROP_ENV_SETTINGS:
      data = g_value_get_pointer (value); 
      priv->env = (EnvironmentSettings*) value; 
      break;
    case PROP_PROJECT:
      data = g_value_get_pointer (value); 
      priv->project = (ProjectSettings*) value; 
      break;
    case PROP_PLUGINS:
      data = g_value_get_pointer (value); 
      priv->pluginlist = (OpenLDevPluginList*) value; 
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

static void
openldev_init (OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);

  // Set properties of main window
  gtk_window_set_title (GTK_WINDOW (openldev), 
                        g_strdup_printf ("%s%s", "OpenLDev ", OPENLDEV_VERSION));
  
  // Create settings objects
  priv->project = openldev_project_settings_new ("");
  priv->env = openldev_environment_settings_new ();
  priv->editor = openldev_editor_new ();
  
  // Call functions to add main window widgets & show them all
  gtk_window_set_default_size (GTK_WINDOW (openldev), priv->env->width,
                               priv->env->height);
  
  openldev_initialize_objects (openldev);
  priv->pluginlist = openldev_load_plugins_all (openldev);
  gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->output), 0);
  openldev_manage_menu_items (openldev);
  openldev_initialize_gconf (openldev);
}

GtkWidget*
openldev_new ()
{
  return GTK_WIDGET (g_object_new (openldev_get_type (), NULL));
}

void 
openldev_initialize_objects (OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);

  priv->vpaned = gtk_vpaned_new ();
  priv->hpaned = gtk_hpaned_new ();
  priv->vbox = gtk_vbox_new (FALSE, 0);
  priv->statusbar = openldev_status_bar_new ();
  priv->output = output_manager_new ();
  priv->files = file_manager_new ();
  priv->sidebar = sidebar_new ();
  priv->findtoolbar = openldev_find_toolbar_new ();
  priv->gototoolbar = openldev_goto_toolbar_new ();
  priv->menus = menu_handle_new ();
  
  gtk_widget_set_size_request (FILE_BROWSER (SIDEBAR (priv->sidebar)->filebrowser)->swin, 
                               priv->env->sidebar_width, 0);
  gtk_widget_set_size_request (OUTPUT (priv->output)->terminal, 0, 
                               priv->env->outputmanager_height);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (priv->output), GTK_POS_LEFT);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (priv->output), TRUE);
  
  // Add all of the widgets & show them
  openldev_populate_window (openldev);
  gtk_widget_show_all (GTK_WIDGET (openldev));
  
  openldev_find_toolbar_show_error (priv->findtoolbar, FALSE);
  openldev_goto_toolbar_show_error (priv->gototoolbar, FALSE);
  gtk_widget_hide (priv->findtoolbar->hbox);
  gtk_widget_hide (priv->gototoolbar->hbox);
}

void 
openldev_populate_window (OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);

  openldev_create_menus_and_toolbar (openldev);
  
  // Add menu & toolbar to main vbox
  gtk_box_pack_start (GTK_BOX (priv->vbox), priv->menus->menubar, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (priv->vbox), priv->menus->toolbar, FALSE, FALSE, 0);
  gtk_paned_pack1 (GTK_PANED (priv->hpaned), priv->sidebar, FALSE, TRUE);
  gtk_paned_pack2 (GTK_PANED (priv->hpaned), priv->files, TRUE, TRUE);
  gtk_paned_pack1 (GTK_PANED (priv->vpaned), priv->hpaned, TRUE, TRUE);
  gtk_paned_pack2 (GTK_PANED (priv->vpaned), priv->output, FALSE, TRUE);

  // Add scrolled_win3 to the second vbox and finish off with the statusbar
  gtk_box_pack_start (GTK_BOX (priv->vbox), priv->vpaned, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (priv->vbox), priv->gototoolbar->hbox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (priv->vbox), priv->findtoolbar->hbox, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (priv->vbox), priv->statusbar->appbar, FALSE, FALSE, 0);

  // Add v' & h' boxes to the main window
  gtk_container_add (GTK_CONTAINER (openldev), priv->vbox);
}

void
openldev_populate_treeview (OpenLDev *openldev)
{ 
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);

  file_browser_populate (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (priv->sidebar))), 
                         openldev_project_settings_get_location (priv->project));
}

void 
openldev_create_menus_and_toolbar (OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);

  // Create menu actions and icons
  menu_handle_create_icon_factory ();

  gtk_action_group_add_actions (priv->menus->group, entries, 
                                NUM_ENTRIES, (gpointer) openldev);
  gtk_action_group_add_toggle_actions (priv->menus->group, toggle_entries, 
                                       NUM_ENTRIES_TOGGLE, (gpointer) openldev);
  menu_handle_uimanager_init (priv->menus);
  gtk_window_add_accel_group (GTK_WINDOW (openldev), 
                              gtk_ui_manager_get_accel_group (priv->menus->uimanager));

  menu_handle_set_sensitive (priv->menus->uimanager, "/MenuBar/File/FileClose", FALSE);
  menu_handle_set_sensitive (priv->menus->uimanager, "/MenuBar/Project", FALSE);
  menu_handle_set_sensitive (priv->menus->uimanager, "/MenuBar/Build", FALSE);

  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_ui_manager_get_widget 
                 (priv->menus->uimanager, "/MenuBar/View/ViewLineNumbering")), 
                  priv->env->line_numbers);
  
  openldev_populate_recent_files (openldev);
}

void
openldev_populate_recent_files (OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);

  // Hide the recent files until they are needed or populate them
  for (int i=0; i < 5; i++)
  {
    gchar buff[2], *recent;
    sprintf (buff, "%d", i+1);
    recent = g_strdup_printf ("%s%s", "/MenuBar/File/FileRecent", buff);
    
    if (g_slist_length (priv->env->recent_files) <= i)
      gtk_widget_hide (gtk_ui_manager_get_widget (priv->menus->uimanager, recent));
    else
    {
      gchar *tooltip = g_strdup_printf ("%s%s%s", "Open \"", (gchar*) g_slist_nth_data 
                                        (priv->env->recent_files, i), "\"");

      gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (gtk_ui_manager_get_widget 
                          (priv->menus->uimanager, recent)))),
                          g_strconcat (buff, ". ", openldev_environment_get_recent_short 
                          (priv->env, i), NULL));
      gtk_tooltips_set_tip (priv->menus->tooltips, gtk_ui_manager_get_widget 
                            (priv->menus->uimanager, recent),
                            tooltip, tooltip);
      g_free (tooltip);
    }
    
    g_signal_connect (G_OBJECT (gtk_ui_manager_get_widget 
                      (priv->menus->uimanager, recent)), "activate",
                      G_CALLBACK (openldev_on_recent_file_activated), 
                      (gpointer) openldev);
    
    g_free (recent);
  }
}

// Quit the program
void 
openldev_file_quit (GtkMenuItem* menuitem, 
                    OpenLDev *openldev) 
{ 
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  gboolean is_modified = FALSE;
  
  for (int i=0; i < FILE_MANAGER (priv->files)->tab_count; i++)
    if (FILE_TAB (FILE_MANAGER (priv->files)->tabs[i])->modified)
      is_modified = TRUE;
  
  if (!is_modified)
    gtk_widget_destroy (GTK_WIDGET (openldev));
  else if (openldev_question (_("You have unsaved files! Are you sure you want to close?")))
    gtk_widget_destroy (GTK_WIDGET (openldev));

}

void 
openldev_on_recent_file_activated (GtkMenuItem *item,
                                   OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  GtkWidget *label = gtk_bin_get_child (GTK_BIN (item));
  const gchar *text = gtk_label_get_text (GTK_LABEL (label));
  gint choice = text[0] - 49;
  
  if (choice >= 0 && choice < 5)
    openldev_open_file (openldev, 
                        (gchar*) g_slist_nth_data (priv->env->recent_files, choice));
}

void 
openldev_open_file_browser (GtkMenuItem *item,
                            OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                (sidebar_get_file_browser (SIDEBAR (priv->sidebar))));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gchar *fn, *file;
    gtk_tree_model_get (model, &iter, 1, &fn, -1);
   
    // If the text is not a group title, open it
    file = g_strdup_printf ("%s%s%s", FILE_BROWSER (sidebar_get_file_browser 
                            (SIDEBAR (priv->sidebar)))->base,
                            FILE_BROWSER (sidebar_get_file_browser 
                            (SIDEBAR (priv->sidebar)))->current, fn);
    
    if (g_ascii_strcasecmp (fn, "..") == 0)
      file_browser_move_up (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (priv->sidebar))));
    else if (g_file_test (file, G_FILE_TEST_IS_DIR))
      file_browser_move (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (priv->sidebar))), fn);
    else
      openldev_open_file (openldev, file);
      
    g_free (fn);
    g_free (file);
  }
}

void 
openldev_jump_to_build_output (GtkMenuItem *item, 
                               OpenLDev *openldev)
{ 
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                         (OUTPUT (priv->output)->build));
  GtkTreeModel *model;
  GtkTreePath *path = gtk_tree_path_new ();
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    string dstr;
    gchar *file;
    int linenumber;
    gtk_tree_model_get (model, &iter, 0, &file, -1);
    
    // It the output is an error or a warning..
    if (openldev_is_error_or_warning (file))
    {
      gchar *dir;
      string temp = file;
      linenumber = openldev_get_file_name_and_line (temp) - 1;
      
      do
      {
        path = gtk_tree_model_get_path (model, &iter);
        gtk_tree_path_prev (path);
        gtk_tree_model_get_iter (model, &iter, path);
        gtk_tree_model_get (model, &iter, 0, &dir, -1);
        dstr = string (dir);
      } while (dstr.find ("make[", 0) == string::npos);
        
      dstr = dir;
      dstr.erase (0, dstr.find ("Entering", 0) + 20);
      dstr = dstr.substr (0, dstr.find ("'", 0));

      openldev_open_file (openldev, g_strconcat (dstr.c_str(), "/", temp.c_str(), NULL));
        
      // Move the cursor to the line of the error & scroll to the cursor
      GtkTextBuffer *buffer = GTK_TEXT_BUFFER (file_manager_get_current_buffer
                                              (FILE_MANAGER (priv->files)));
      GtkTextIter iter;

      gtk_text_buffer_get_iter_at_line (buffer, &iter, linenumber);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_insert (buffer), &iter);
      gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_selection_bound (buffer), &iter);
      file_manager_scroll_to_mark (FILE_MANAGER (priv->files));
      gtk_widget_grab_focus (GTK_WIDGET (file_manager_get_current_view 
                             (FILE_MANAGER (priv->files))));
      g_free (dir);
    }
      
    g_free (file);
  }
}

void 
openldev_jump_to_search_output (GtkMenuItem *item, 
                                OpenLDev *openldev)
{ 
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW 
                                         (OUTPUT (priv->output)->search));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gchar *file;
    int linenumber;
    gtk_tree_model_get (model, &iter, 0, &file, -1);
      
    // Open the file the error was found in
    string temp = file;
    linenumber = openldev_get_file_name_and_line (temp) - 1;
    file = g_strconcat (openldev_project_settings_get_location (priv->project), 
                        temp.c_str(), NULL);
    openldev_open_file (openldev, file);
    
    // Move the cursor to the line of the error & scroll to the cursor
    GtkTextIter iter;
    GtkTextBuffer *buffer = GTK_TEXT_BUFFER (file_manager_get_current_buffer 
                                            (FILE_MANAGER (priv->files)));
      
    gtk_text_buffer_get_iter_at_line (buffer, &iter, linenumber);
    gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_insert (buffer), &iter);
    gtk_text_buffer_move_mark (buffer, gtk_text_buffer_get_selection_bound( buffer), &iter);
    file_manager_scroll_to_mark (FILE_MANAGER (priv->files));
    gtk_widget_grab_focus (GTK_WIDGET (file_manager_get_current_view 
                           (FILE_MANAGER (priv->files))));
     
    g_free (file);
  }
}

// Menu handler functions and signal handlers that do nothing but redirect

void openldev_on_find_and_replace (GtkWidget *widget, OpenLDev *openldev)
{ openldev_find_and_replace (openldev); }

void openldev_refresh_file_browser (GtkMenuItem *item, OpenLDev *openldev)
{ 
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  file_browser_refresh (FILE_BROWSER (sidebar_get_file_browser (SIDEBAR (priv->sidebar)))); 
}

void openldev_clear_build_output (GtkMenuItem *item, OpenLDev *openldev)
{ 
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  output_build_clear (OUTPUT_BUILD (OUTPUT (priv->output)->build)); 
}

void openldev_clear_search_output (GtkMenuItem *item, OpenLDev *openldev)
{
  OpenLDevPrivate *priv = OPENLDEV_GET_PRIVATE (openldev);
  output_list_clear (LIST (OUTPUT (priv->output)->search)); 
}

void openldev_copy_output (GtkMenuItem *item, OpenLDev *openldev)
{ openldev_menu_edit_copy (openldev); }

void openldev_file_close_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_close_project (openldev); }

void openldev_file_import_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_import_project (openldev); }

void openldev_file_new_file (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_new_file (openldev); }

void openldev_file_new_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_new_project (openldev); }

void openldev_file_open (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_open (openldev); }

void openldev_file_open_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_open_project (openldev); }

void openldev_file_revert (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_revert (openldev); }

void openldev_file_save (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_save (openldev); }

void openldev_file_save_all (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_save_all (openldev); }

void openldev_file_save_as (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_save_as (openldev); }

void openldev_file_print (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_print (openldev); }

void openldev_file_print_preview (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_file_print_preview (openldev); }

void openldev_edit_copy (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_copy (openldev); }

void openldev_edit_cut (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_cut (openldev); }

void openldev_edit_deselect (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_deselect (openldev); }

void openldev_edit_paste (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_paste (openldev); }

void openldev_edit_preferences (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_preferences (openldev); }

void openldev_edit_redo (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_redo (openldev); }

void openldev_edit_select_all (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_select_all (openldev); }

void openldev_edit_undo (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_undo (openldev); }

void openldev_edit_find (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_find (openldev); }

void openldev_edit_find_next (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_find_next (openldev); }

void openldev_edit_goto_line (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_goto_line (openldev); }

void openldev_edit_indent (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_indent (openldev); }

void openldev_edit_unindent (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_edit_unindent (openldev); }

void openldev_project_info (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_project_info (openldev); }

void openldev_project_preferences (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_project_preferences (openldev); }

void openldev_build_auto_generate (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_auto_generate (openldev); }

void openldev_build_clean (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_clean (openldev); }

void openldev_build_compile (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_compile (openldev); }

void openldev_build_configure (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_configure (openldev); }

void openldev_build_execute (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_execute (openldev); }

void openldev_build_project (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_project (openldev); }

void openldev_build_stop (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_build_stop (openldev); }

void openldev_view_browser (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_browser (openldev); }

void openldev_view_line_numbering (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_line_numbering (openldev); }

void openldev_view_output (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_output (openldev); }

void openldev_view_statusbar (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_statusbar (openldev); }

void openldev_view_toolbar (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_view_toolbar (openldev); }

void openldev_help_about (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_help_about (openldev); }

void openldev_help_documentation (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_help_documentation (openldev); }

void openldev_help_report_bug (GtkMenuItem *menuitem, OpenLDev *openldev)
{ openldev_menu_help_report_bug (openldev); }
